# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

testCallbackOrdering <- function() {
    invisible(.Call(`_later_testCallbackOrdering`))
}

log_level <- function(level) {
    .Call(`_later_log_level`, level)
}

using_ubsan <- function() {
    .Call(`_later_using_ubsan`)
}

execLater_fd <- function(callback, readfds, writefds, exceptfds, timeoutSecs, loop_id) {
    .Call(`_later_execLater_fd`, callback, readfds, writefds, exceptfds, timeoutSecs, loop_id)
}

fd_cancel <- function(xptr) {
    .Call(`_later_fd_cancel`, xptr)
}

setCurrentRegistryId <- function(id) {
    invisible(.Call(`_later_setCurrentRegistryId`, id))
}

getCurrentRegistryId <- function() {
    .Call(`_later_getCurrentRegistryId`)
}

deleteCallbackRegistry <- function(loop_id) {
    .Call(`_later_deleteCallbackRegistry`, loop_id)
}

notifyRRefDeleted <- function(loop_id) {
    .Call(`_later_notifyRRefDeleted`, loop_id)
}

createCallbackRegistry <- function(id, parent_id) {
    invisible(.Call(`_later_createCallbackRegistry`, id, parent_id))
}

existsCallbackRegistry <- function(id) {
    .Call(`_later_existsCallbackRegistry`, id)
}

list_queue_ <- function(id) {
    .Call(`_later_list_queue_`, id)
}

execCallbacks <- function(timeoutSecs, runAll, loop_id) {
    .Call(`_later_execCallbacks`, timeoutSecs, runAll, loop_id)
}

idle <- function(loop_id) {
    .Call(`_later_idle`, loop_id)
}

ensureInitialized <- function() {
    invisible(.Call(`_later_ensureInitialized`))
}

execLater <- function(callback, delaySecs, loop_id) {
    .Call(`_later_execLater`, callback, delaySecs, loop_id)
}

cancel <- function(callback_id_s, loop_id) {
    .Call(`_later_cancel`, callback_id_s, loop_id)
}

nextOpSecs <- function(loop_id) {
    .Call(`_later_nextOpSecs`, loop_id)
}

