% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/later.R
\name{list_queue}
\alias{list_queue}
\title{Get the contents of an event loop, as a list}
\usage{
list_queue(loop = current_loop())
}
\description{
This function is for debugging only.
}
\keyword{internal}
